/*******************************************************************************
 * Copyright (C) 2018, OpenRefine contributors
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************/

package com.google.refine.model;

import org.testng.annotations.Test;

import com.google.refine.util.TestUtils;

public class ColumnGroupTests {

    String json = "{"
            + "\"startColumnIndex\":2,"
            + "\"columnSpan\":3,"
            + "\"keyColumnIndex\":1"
            + "}";

    @Test
    public void serializeColumnGroup() throws Exception {
        TestUtils.isSerializedTo(ColumnGroup.load(json), json);
    }

    @Test
    public void serializeColumnGroupWithSubgroups() throws Exception {
        ColumnGroup cg = new ColumnGroup(2, 3, 1);
        ColumnGroup subCg = new ColumnGroup(2, 2, 1);
        cg.subgroups.add(subCg);
        String fullJson = "{"
                + "\"startColumnIndex\":2,"
                + "\"columnSpan\":3,"
                + "\"keyColumnIndex\":1,"
                + "\"subgroups\":[{"
                + "   \"startColumnIndex\":2,"
                + "   \"columnSpan\":2,"
                + "   \"keyColumnIndex\":1"
                + "}]"
                + "}";
        TestUtils.isSerializedTo(cg, json, true);
        TestUtils.isSerializedTo(cg, fullJson, false);
    }
}
