const foodMini = require('./food.mini.js');
const foodSmall = require('./food.small.js');
const foodSort = require('./food.sort.js');

const fixtures = {
  'food.mini': foodMini,
  'food.small': foodSmall,
  'food.sort': foodSort,
};

export default fixtures;
