<div class="data-table-topic-popup-header">
  <button class="button" title="Match item to this cell" bind="matchButton"></button>
  <button class="button" title="Match item to all visible cells with same content" bind="matchSimilarButton"></button>
  <button class="button" title="Cancel" bind="cancelButton"></button>
</div>
