<div class="dialog-frame" style="min-width: 700px;">
  <div class="dialog-border">
    <div class="dialog-header" bind="dialogHeader"></div>
    <form bind="form">
    <div class="dialog-body" bind="dialogBody">
      <div class="grid-layout layout-normal layout-full"><table cols="2" style="width: 700px;">
        <tr>
          <td width="1%" style="white-space: pre;"><label for="columnNameInputId" bind="or_views_newCol"></label></td>
          <td><input bind="columnNameInput" type="text" id="columnNameInputId" size="40" /></td>

          <td width="1%" style="white-space: pre;"><label for="throttleDelayInputId" bind="or_views_throttle"></label></td>
          <td><input bind="throttleDelayInput" id="throttleDelayInputId" type="number" size="10" value="500" /> <span bind="or_views_milli"></span></td>
        </tr>
        <tr>
          <td width="1%" style="white-space: pre;" bind="or_views_onErr"></td>
          <td>
            <input type="radio" name="dialog-onerror-choice" value="set-to-blank" checked id="$add-column-error-set-to-blank"/>
              <label for="$add-column-error-set-to-blank" bind="or_views_setBlank"></label>
            <input type="radio" name="dialog-onerror-choice" value="store-error" id="$add-column-error-store-error" />
              <label for="$add-column-error-store-error" bind="or_views_storeErr"></label></td>
	   <td colspan="2">
	     <input type="checkbox" name="dialog-cache-responses" id="$add-column-cache-responses" checked="checked" />
		<label for="$add-column-cache-responses" bind="or_views_cacheResponses"></label></td>
        </tr>
        <tr><td colspan="4">
          <details>
            <summary bind="or_views_httpHeaders"></summary>
            $HTTP_HEADERS_WIDGET$
          </details>
          <span class="toggle-text" bind=""></span>
        </td></tr>
        <tr><td colspan="4"><h3><span bind="or_views_urlFetch"></span></h3></td></tr>
        <tr><td colspan="4">$EXPRESSION_PREVIEW_WIDGET$</td></tr>
      </table></div>
    </div>
    <div class="dialog-footer" bind="dialogFooter">
      <button class="button button-primary" type="submit" bind="okButton"></button>
      <button class="button" bind="cancelButton"></button>
    </div>
    </form>
  </div>
</div>
