<div class="column-header-title"><button class="column-header-menu" bind="dropdownMenu"></button><span class="column-header-name" bind="nameContainer"></span></div>
<div style="display:none;" bind="reconStatsContainer"></div>