<div class="dialog-frame" style="width: 400px">
    <form bind="form">
        <div class="dialog-border">
            <div class="dialog-header" bind="dialogHeader"></div>
            <div class="dialog-body" bind="dialogBody">
                <div class="grid-layout layout-normal layout-full">
                    <div style="padding: 10px">
                        <input aria-label="" bind="columnNameInput" size="40" autofocus/>
                    </div>
                </div>
            </div>
            <div class="dialog-footer" bind="dialogFooter">
                <button type="submit" class="button button-primary" bind="okButton"/>
                <button class="button" bind="cancelButton"></button>
            </div>
        </div>
    </form>
</div>
