<div class="dialog-frame" style="min-width: 600px;">
    <div class="dialog-border">
        <div class="dialog-header" bind="dialogHeader"></div>
        <div class="dialog-body" bind="dialogBody">
            <div class="grid-layout layout-tight layout-full"><table>
                <tr>
                    <td><span bind="or_views_warning"></span></td>
                </tr>
            </table>
            </div>
        </div>
        <div class="dialog-footer" bind="dialogFooter">
            <button class="button button-primary" bind="okButton"></button>
            <button class="button" bind="cancelButton"></button>
        </div>
    </div>
</div>