/*******************************************************************************
 * MIT License
 * 
 * Copyright (c) 2018 Antonin Delpeuch
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/

package org.openrefine.wikibase.schema;

import org.testng.annotations.Test;
import org.wikidata.wdtk.datamodel.helpers.Datamodel;
import org.wikidata.wdtk.datamodel.interfaces.MonolingualTextValue;

public class WbMonolingualExprTest extends WbExpressionTest<MonolingualTextValue> {

    private WbMonolingualExpr expr = new WbMonolingualExpr(new WbLanguageVariable("column A"),
            new WbStringVariable("column B"));

    @Test
    public void testEvaluateConstant() {
        evaluatesTo(Datamodel.makeMonolingualTextValue("hello", "en"),
                new WbMonolingualExpr(new WbLanguageConstant("en", "English"), new WbStringConstant("hello")));
    }

    @Test
    public void testValidate() {
        hasNoValidationError(new WbMonolingualExpr(new WbLanguageConstant("en", "English"), new WbStringConstant("hello")));
        hasValidationError("No language provided", new WbMonolingualExpr(null, new WbStringConstant("hello")));
        hasValidationError("No text value provided", new WbMonolingualExpr(new WbLanguageConstant("en", "English"), null));
    }

    @Test
    public void testEvaluateVariable() {
        setRow("en", "hello");
        evaluatesTo(Datamodel.makeMonolingualTextValue("hello", "en"), expr);
    }

    @Test
    public void testTrim() {
        setRow("en", " hello ");
        evaluatesTo(Datamodel.makeMonolingualTextValue("hello", "en"), expr);
    }

    @Test
    public void testInvalidLanguageCode() {
        setRow("ueuue", "my label");
        isSkipped(expr);
    }

    @Test
    public void testEmptyLanguageCode() {
        setRow("", "my label");
        isSkipped(expr);
    }

    @Test
    public void testEmptyText() {
        setRow("en", "");
        isSkipped(expr);
    }
}
