
<label for="typeSelectId" bind="or_views_dataType"></label> <select bind="typeSelect" id="typeSelectId">
  <option value="text" bind="or_views_text"></option>
  <option value="number" bind="or_views_number"></option>
  <option value="boolean" bind="or_views_boolean"></option>
  <option value="date" bind="or_views_date"></option>
</select>

<div id="data-table-cell-editor-help" bind="cell_help">
  <span bind="cell_help_text"></span>&nbsp;
</div>

<textarea aria-label="" class="data-table-cell-editor-editor" bind="textarea"></textarea>

<div id="data-table-cell-editor-actions">
  <div class="data-table-cell-editor-action">
    <button class="button button-primary" bind="okButton"></button>
    <div class="data-table-cell-editor-key" bind="or_views_enter"></div>
  </div>
  <div class="data-table-cell-editor-action">
    <button class="button" bind="okallButton"></button>
    <div class="data-table-cell-editor-key" bind="or_views_ctrlEnter"></div>
  </div>
  <div class="data-table-cell-editor-action">
    <button class="button" bind="cancelButton"></button>
    <div class="data-table-cell-editor-key" bind="or_views_esc"></div>
  </div>
</div>
